﻿#include "game.h"
#include "walls.h"
#include "player.h"
#include "../common/sources/system/fileCache.h"
#include "../common/sources/system/cgraphics2d.h"
#include <QDebug>
#include <QPainter>
#include "interface.h"

CGame   game;

//-----------------------------------------------------------------------------------------
CGame::CGame()
{
    characters[0].picture = 0;
    characters[0].name = "ZED";
    characters[0].stats[CCharacter::eStatHealth].value = 10;
    characters[0].stats[CCharacter::eStatHealth].maxValue = 10;
    characters[0].stats[CCharacter::eStatStamina].value = 10;
    characters[0].stats[CCharacter::eStatStamina].maxValue = 10;
    characters[0].stats[CCharacter::eStatMana].value = 10;
    characters[0].stats[CCharacter::eStatMana].maxValue = 10;
    characters[0].bodyObjects[CCharacter::eBodyPartRightHand] = -1;

    characters[1].picture = 1;
    characters[1].name = "STAMM";
    characters[1].stats[CCharacter::eStatHealth].value = 5;
    characters[1].stats[CCharacter::eStatHealth].maxValue = 10;
    characters[1].stats[CCharacter::eStatStamina].value = 5;
    characters[1].stats[CCharacter::eStatStamina].maxValue = 10;
    characters[1].stats[CCharacter::eStatMana].value = 5;
    characters[1].stats[CCharacter::eStatMana].maxValue = 10;
    characters[1].bodyObjects[CCharacter::eBodyPartRightHand] = -1;

    characters[2].picture = 2;
    characters[2].name = "WUUF";
    characters[2].stats[CCharacter::eStatHealth].value = 0;
    characters[2].stats[CCharacter::eStatHealth].maxValue = 10;
    characters[2].stats[CCharacter::eStatStamina].value = 10;
    characters[2].stats[CCharacter::eStatStamina].maxValue = 10;
    characters[2].stats[CCharacter::eStatMana].value = 10;
    characters[2].stats[CCharacter::eStatMana].maxValue = 10;
    characters[2].bodyObjects[CCharacter::eBodyPartRightHand] = -1;

    shadowImage = new QImage(MAINVIEW_WIDTH, MAINVIEW_HEIGHT, QImage::Format_ARGB32);
    shadowImage->fill(QColor(0, 0, 0));
}

// table avec les positions et tailles de chaque mur pour les décors
struct SWallInfos
{
    CVec2   pos;
    CVec2   size;
};

static const SWallInfos   gWallsInfos[WALL_TABLE_HEIGHT][WALL_TABLE_WIDTH][3] =
{
    // Rang 3 (le plus loin)
    {
        {{{0, 0}, {0, 0}}, {{0, 0}, {0, 0}},     {{0, 0}, {0, 0}}     }, // 03
        {{{0, 0}, {0, 0}}, {{ 8, 58}, {36, 48}}, {{0, 0}, {0, 0}}     }, // 13
        {{{0, 0}, {0, 0}}, {{78, 59}, {12, 48}}, {{134, 59}, {12, 48}}}, // 23
        {{{0, 0}, {0, 0}}, {{0, 0}, {0, 0}},     {{180, 58}, {36, 48}}}, // 33
        {{{0, 0}, {0, 0}}, {{0, 0}, {0, 0}},     {{0, 0}, {0, 0}}     }  // 43
    },

    // Rang 2
    {
        {{{-77, 58}, {70, 48}}, {{0, 0}, {0, 0}},     {{0, 0}, {0, 0}}     }, // 02
        {{{  7, 58}, {70, 48}}, {{0, 0}, {0, 0}},     {{0, 0}, {0, 0}}     }, // 12
        {{{ 77, 58}, {70, 48}}, {{60, 52}, {18, 74}}, {{146, 52}, {18, 74}}}, // 22
        {{{146, 58}, {70, 48}}, {{0, 0}, {0, 0}},     {{0, 0}, {0, 0}}     }, // 32
        {{{216, 58}, {70, 48}}, {{0, 0}, {0, 0}},     {{0, 0}, {0, 0}}     }  // 42
    },

    // Rang 1
    {
        {{{0, 0}, {0, 0}},       {{0, 0}, {0, 0}},      {{0, 0}, {0, 0}}      }, // 01
        {{{-46, 52}, {105, 74}}, {{0, 0}, {0, 0}},      {{0, 0}, {0, 0}}      }, // 11
        {{{ 59, 52}, {105, 74}}, {{33, 42}, {27, 111}}, {{164, 42}, {27, 111}}}, // 21
        {{{164, 52}, {105, 74}}, {{0, 0}, {0, 0}},      {{0, 0}, {0, 0}}      }, // 31
        {{{0, 0}, {0, 0}},       {{0, 0}, {0, 0}},      {{0, 0}, {0, 0}}      }  // 41
    },

    // Rang 0 (le plus proche)
    {
        {{{0, 0}, {0, 0}},         {{0, 0}, {0, 0}}, {{0, 0}, {0, 0}}}, // 00
        {{{-127, 42}, {159, 111}}, {{0, 0}, {0, 0}}, {{0, 0}, {0, 0}}}, // 10
        {{{  32, 42}, {159, 111}}, {{0, 0}, {0, 0}}, {{0, 0}, {0, 0}}}, // 20
        {{{ 191, 42}, {159, 111}}, {{0, 0}, {0, 0}}, {{0, 0}, {0, 0}}}, // 30
        {{{0, 0}, {0, 0}},         {{0, 0}, {0, 0}}, {{0, 0}, {0, 0}}}  // 40
    }
};

//-----------------------------------------------------------------------------------------
void CGame::drawWall(QImage* image, CVec2 mapPos, CVec2 tablePos, EWallSide side, bool flip)
{
    CTile*  tile = map.getTile(mapPos);

    if (tile != NULL)
    {
        EWallSide playerSide = player.getWallSide(side);

        if (tile->mWalls[playerSide].getType() != eWallNothing)
        {
            const char* fileName = walls.getFilename(tablePos, side, flip);

            if (fileName[0] != 0)
            {
                // dessine le mur
                QImage  wallGfx = fileCache.getImage(fileName);
                CVec2 gfxPos = walls.getGfxPos(tablePos, side, flip);

                graph2D.drawImage(image, gfxPos, wallGfx, 0, flip);

                // dessine les décors
                CWall*  wall = &tile->mWalls[playerSide];

                if (wall->getType() == eWallSimple)
                {
                    int ornate = ((CParamOrnate*)wall->mParams[0])->mValue;

                    if (ornate != 0)
                    {
                        int tableSide = walls.getTableSide(side, false);

                        if (gWallsInfos[tablePos.y][tablePos.x][tableSide].size.x != 0)
                        {
                            CWalls::COrnateData& ornateData = walls.ornatesDatas[ornate];
                            const SWallInfos*   ornateTabData = &gWallsInfos[tablePos.y][tablePos.x][tableSide];
                            const SWallInfos*   ornate0 = NULL;
                            QString ornateFileName;
                            CVec2   ornatePos;
                            bool    ornateFlip = false;

                            switch(side)
                            {
                                case eWallSideUp:
                                    ornate0 = &gWallsInfos[3][2][tableSide];
                                    ornateFileName = ornateData.imageFront;
                                    ornatePos = ornateData.posFront;
                                break;

                                case eWallSideLeft:
                                    ornate0 = &gWallsInfos[2][2][1];
                                    ornateFileName = walls.ornatesDatas[ornate].imageSide;
                                    ornatePos = ornateData.posSide;
                                break;

                                case eWallSideRight:
                                    ornate0 = &gWallsInfos[2][2][1];
                                    ornateFileName = walls.ornatesDatas[ornate].imageSide;
                                    ornatePos = ornateData.posSide;
                                    ornateFlip = true;
                                break;

                                default:
                                    break;
                            }

                            // calcule la position et le facteur d'échelle et dessine le décor dans une image temporaire
                            float   scale = (float)ornateTabData->size.y / 111.0f;
                            QImage  ornateImage = fileCache.getImage(ornateFileName.toLocal8Bit().constData());
                            QSize   ornateSize = ornateImage.size() * scale;
                            ornatePos = (ornatePos - ornate0->pos);
                            ornatePos.x = ornatePos.x * ornateTabData->size.x / ornate0->size.x;
                            ornatePos.y = ornatePos.y * ornateTabData->size.y / ornate0->size.y;

                            if (ornateFlip == true)
                                ornatePos.x = ornateTabData->size.x - ornatePos.x - ornateSize.width();

                            ornatePos += ornateTabData->pos;

                            QImage  tempImage(ornateSize, QImage::Format_ARGB32);
                            tempImage.fill(QColor(0, 0, 0, 0));
                            graph2D.drawImageScaled(&tempImage, CVec2(), ornateImage, scale, ornateFlip);

                            // assombrit le décor en fonction de sa distance
                            static const float shadowLevels[] = {0.0f, 0.2f, 0.4f};
                            float shadow;

                            if (side == eWallSideUp)
                                shadow = shadowLevels[WALL_TABLE_HEIGHT - 1 - tablePos.y];
                            else
                                shadow = shadowLevels[WALL_TABLE_HEIGHT - 2 - tablePos.y];

                            QPainter painter(&tempImage);
                            painter.setOpacity(shadow);
                            painter.setCompositionMode(QPainter::CompositionMode_SourceAtop);
                            painter.drawImage(QPoint(0, 0), *shadowImage);
                            painter.setCompositionMode(QPainter::CompositionMode_SourceOver);
                            painter.setOpacity(1.0f);
                            painter.end();

                            // dessine les décors à l'écran.
                            graph2D.drawImage(image, ornatePos, tempImage, 0, false, QRect(0, 33, MAINVIEW_WIDTH, MAINVIEW_HEIGHT));
                        }
                    }
                }
            }
        }
    }
}

//-----------------------------------------------------------------------------------------
QImage CGame::displayMainView()
{
    QImage image(SCREEN_WIDTH,
                 SCREEN_HEIGHT,
                 QImage::Format_RGBA8888);

	// efface l'écran
    image.fill(QColor(0, 0, 0));

    if (interface.isInInventory() == false)
    {
		// dessine le sol et le plafond
        QImage  floorGfx = fileCache.getImage("gfx/3DView/Floor.png");
        QImage  ceilingGfx = fileCache.getImage("gfx/3DView/Ceiling.png");

        bool flip = (player.pos.x + player.pos.y + player.dir) & 1;

        graph2D.drawImage(&image, CVec2(0, 33), ceilingGfx, 0, flip);
        graph2D.drawImage(&image, CVec2(0, 72), floorGfx, 0, flip);

		// dessine les murs
        CVec2   tablePos;
        CVec2   localPos;
        CVec2   mapPos;

        for (int y = 0; y < WALL_TABLE_HEIGHT; ++y)
            for (int x = 0; x < WALL_TABLE_WIDTH; ++x)
            {
                tablePos = walls.getDrawSequence(CVec2(x, y));
                localPos = walls.getMapPos(tablePos);
                mapPos = player.getPosFromLocal(localPos);

                drawWall(&image, mapPos, tablePos, eWallSideUp, flip);
                drawWall(&image, mapPos, tablePos, eWallSideLeft, flip);
                drawWall(&image, mapPos, tablePos, eWallSideRight, flip);
            }
    }

    interface.display(&image);

    return image;
}
