#include "interface.h"
#include "game.h"
#include "../common/sources/system/cgraphics2d.h"
#include "../common/sources/system/filecache.h"

CInterface   interface;

static const int   championsColors[4][3] =
{
    {  0, 222,   0},
    {255, 255,   0},
    {255,   0,   0},
    {  0,   0, 255}
};

//------------------------------------------------------------
CInterface::CInterface()
{
    mainState = eMainGame;
    weaponsAreaState = eWeaponsAreaWeapons;
    currentChampion = -1;
    currentWeapon = -1;

    for (int i = 0; i < 4; ++i)
    {
        weaponCoolDown[i] = new QTimer();
        weaponCoolDown[i]->setSingleShot(true);
    }
}

//------------------------------------------------------------
CInterface::~CInterface()
{
    for (int i = 0; i < 4; ++i)
        delete weaponCoolDown[i];
}

//------------------------------------------------------------
void    CInterface::display(QImage* image)
{
    if (isInInventory() == true)
        drawInventory(image);

    for (int i = 0; i < 4; ++i)
        drawChampion(image, i);

    drawSpells(image);
    drawWeaponsArea(image);
    drawArrows(image);
}

//------------------------------------------------------------
bool    CInterface::isInInventory()
{
    return (mainState == eMainInventory);
}

//------------------------------------------------------------
bool    CInterface::isChampionEmpty(int num)
{
    CCharacter* c = &game.characters[num];
    return c->picture == -1;
}

//------------------------------------------------------------
bool    CInterface::isChampionDead(int num)
{
    CCharacter* c = &game.characters[num];
    return c->isDead();
}

//------------------------------------------------------------
bool    CInterface::isWeaponEmpty(int num)
{
    return (isChampionDead(num) || isChampionEmpty(num));
}

//------------------------------------------------------------
bool    CInterface::isWeaponGreyed(int num)
{
    return weaponCoolDown[num]->isActive();
}

//------------------------------------------------------------
bool    CInterface::isArrowsGreyed()
{
    return isInInventory();
}

void    CInterface::drawBar(QImage* image, int num, CVec2 pos, CCharacter::EStatsNames statName)
{
    CCharacter::SStat   stat = getStat(num, statName);

    pos.y = 27 - (25 * stat.value) / stat.maxValue;
    CRect   rect(pos, CVec2(pos.x + 3, 26));
    QColor color(championsColors[num][0],
                 championsColors[num][1],
                 championsColors[num][2]);
    graph2D.rect(image, rect, color, true);
}

//------------------------------------------------------------
void    CInterface::drawChampion(QImage* image, int num)
{
    if (isChampionEmpty(num) == false)
    {
        if (isChampionDead(num) == true)
        {
            QImage  champBg = fileCache.getImage("gfx/interface/DeadChampInfo.png");
            CVec2   pos(69 * num, 0);
            graph2D.drawImage(image, pos, champBg);
        }
        else
        {
            QImage  champBg = fileCache.getImage("gfx/interface/ChampInfo.png");
            CVec2   pos(69 * num, 0);
            graph2D.drawImage(image, pos, champBg);

            if (isInInventory() == true && currentChampion == num)
            {
                // draw head
            }
            else
            {
                // draw hands
            }

            // bars
            drawBar(image, num, CVec2(69 * num + 46, 2), CCharacter::eStatHealth);
            drawBar(image, num, CVec2(69 * num + 53, 2), CCharacter::eStatStamina);
            drawBar(image, num, CVec2(69 * num + 60, 2), CCharacter::eStatMana);
        }
    }
}

//------------------------------------------------------------
void    CInterface::drawSpells(QImage* image)
{
    QImage  spellsBg = fileCache.getImage("gfx/interface/SpellsArea.png");
    CVec2   pos(233, 50);
    graph2D.drawImage(image, pos, spellsBg);
}

//------------------------------------------------------------
void    CInterface::drawWeaponsArea(QImage* image)
{
    if (weaponsAreaState == eWeaponsAreaWeapons)
    {
        for (int i = 0; i < 4; ++i)
            drawWeapon(image, i);
    }
    else if (weaponsAreaState == eWeaponsAreaAttacks)
    {
        QImage  spellsBg = fileCache.getImage("gfx/interface/WeaponsActions.png");
        CVec2   pos(233, 77);
        graph2D.drawImage(image, pos, spellsBg);
    }
    else
    {
        QImage  spellsBg = fileCache.getImage("gfx/interface/DamageDone.png");
        CVec2   pos(232, 77);
        graph2D.drawImage(image, pos, spellsBg);
    }
}

//------------------------------------------------------------
void    CInterface::greyRectangle(QImage* image, CRect rect)
{
    for (int y = rect.tl.y; y <= rect.br.y; ++y)
        for (int x = rect.tl.x; x <= rect.br.x; ++x)
            if (((x + y) & 1) == 0)
                image->setPixel(QPoint(x, y), 0);
}

//------------------------------------------------------------
void    CInterface::drawWeapon(QImage* image, int num)
{
    if (isWeaponEmpty(num) == false)
    {
        CRect   rect(CVec2(233 + 22 * num, 86),
                     CVec2(252 + 22 * num, 120));
        graph2D.rect(image, rect, MAIN_INTERFACE_COLOR, true);

        if (isWeaponGreyed(num) == true)
            greyRectangle(image, rect);
    }
}

//------------------------------------------------------------
void    CInterface::drawArrows(QImage* image)
{
    QImage  arrowsBg = fileCache.getImage("gfx/interface/MovementArrows.png");
    CVec2   pos(233, 124);
    graph2D.drawImage(image, pos, arrowsBg);

    if (isArrowsGreyed() == true)
    {
        CRect   rect(CVec2(233, 124),
                     CVec2(319, 168));
        greyRectangle(image, rect);
    }
}

//------------------------------------------------------------
void    CInterface::drawInventory(QImage* image)
{

}

//------------------------------------------------------------
int    CInterface::getWeapon(int num)
{
    CCharacter* c = &game.characters[num];
    return c->bodyObjects[CCharacter::eBodyPartRightHand];
}

//------------------------------------------------------------
CCharacter::SStat   CInterface::getStat(int num, CCharacter::EStatsNames statName)
{
    CCharacter* c = &game.characters[num];
    return c->stats[statName];
}
